//
//  AMOutlineView.m
//  A subclass of NSOutlineView
//
//  Created by Andrew Merenbach on 1/1/07.
//  Copyright 2007 Andrew Merenbach. All rights reserved.
//

#import "AMOutlineView.h"

#define AMSpaceCharacter @" "


@implementation AMOutlineView

- (id)init {
	if (nil != (self = [super init])) {
		AM_spaceBarAdds = NO;
		AM_returnKeyAdds = NO;
		AM_deleteKeyRemoves = NO;
		
		AM_spaceBarSendsDoubleAction = NO;
		AM_returnKeySendsDoubleAction = NO;
	}
	return self;
}

- (id)initWithFrame:(NSRect)frameRect {
	if (nil != (self = [super initWithFrame:frameRect])) {
		AM_spaceBarAdds = NO;
		AM_returnKeyAdds = NO;
		AM_deleteKeyRemoves = NO;
		
		AM_spaceBarSendsDoubleAction = NO;
		AM_returnKeySendsDoubleAction = NO;
	}
	return self;
}

- (id)initWithCoder:(NSCoder *)aDecoder {
	if (nil != (self = [super initWithCoder:aDecoder])) {
		AM_spaceBarAdds = NO;
		AM_returnKeyAdds = NO;
		AM_deleteKeyRemoves = NO;
		
		AM_spaceBarSendsDoubleAction = NO;
		AM_returnKeySendsDoubleAction = NO;
	}
	return self;
}

@end

@implementation AMOutlineView (DLKeyDownOperations)

- (void)keyDown:(NSEvent *)event {
    consumedKeyDown = NO;
    [self interpretKeyEvents:[NSArray arrayWithObject:event]]; 
    if (!consumedKeyDown) [super keyDown:event];
}

- (void)doCommandBySelector:(SEL)selector {
    consumedKeyDown = [self tryToPerform:selector with:nil]; 
}

/* credit to Daryn on Cocoa-dev */
- (void)scrollToRow:(int)row selectRow:(BOOL)select {
	NSIndexSet *indices = [NSIndexSet indexSetWithIndex:row];
	if (select) [self selectRowIndexes:indices byExtendingSelection:NO];
	[self scrollRowToVisible:row];
}

- (void)scrollLineUp:(id)sender {
	int row = [self selectedRow];
	if (row < 0) row = [self numberOfRows];
	if (--row >= 0) [self scrollToRow:row selectRow:YES];
}

- (void)scrollLineDown:(id)sender {
	int row = [self selectedRow];
	int numberOfRows = [self numberOfRows];
	if (row < 0) row = (-1);
	if (++row < numberOfRows) [self scrollToRow:row selectRow:YES];
}

- (void)moveDown:(id)sender {
	[self scrollLineDown:sender];
}

- (void)moveUp:(id)sender {
	[self scrollLineUp:sender];
}

- (void)scrollToBeginningOfDocument:(id)sender {
	int numberOfRows = [self numberOfRows];
	if (numberOfRows) [self scrollToRow:0 selectRow:NO];
}

- (void)scrollToEndOfDocument:(id)sender {
	int numberOfRows = [self numberOfRows];
	if (numberOfRows) [self scrollToRow:(numberOfRows - 1) selectRow:NO];
}

@end

@implementation AMOutlineView (DataMethods)

- (void)delete:(id)sender {
	if ([self deleteKeyRemoves]) {
		id dataSource = [self dataSource];
		if ([dataSource respondsToSelector:@selector(remove:)]) {
			[dataSource performSelector:@selector(remove:) withObject:self];
		}
	}
}

- (void)deleteBackward:(id)sender {
	[self delete:sender];
}

- (void)deleteForward:(id)sender {
	[self delete:sender];
}

- (void)insertNewline:(id)sender {
	if ([self returnKeySendsDoubleAction] && ![self returnKeyAdds]) {
		[self sendAction:[self doubleAction] to:[self target]];
	}
	/*else if ([self returnKeyAdds] && ![self returnKeySendsDoubleAction]) {
		id dataSource = [self dataSource];
		if ([dataSource respondsToSelector:@selector(add:)]) {
			[dataSource performSelector:@selector(add:) withObject:self];
		}
	}*/
	else {
		[super insertNewline:sender];
	}
}

- (void)insertText:(id)aString {
	if (![aString isEqualToString:AMSpaceCharacter]) {
		[super insertText:aString];
	}
	else {
		if ([self spaceBarSendsDoubleAction] && ![self spaceBarAdds]) {
			[self sendAction:[self doubleAction] to:[self target]];
		}
		/*else if ([self spaceBarAdds] && ![self spaceBarSendsDoubleAction]) {
			id dataSource = [self dataSource];
			if ([dataSource respondsToSelector:@selector(add:)]) {
				[dataSource performSelector:@selector(add:) withObject:self];
			}
		}*/
	}
}

@end

@implementation AMOutlineView (Accessors)

- (BOOL)spaceBarAdds {
	return AM_spaceBarAdds;
}

- (void)setSpaceBarAdds:(BOOL)flag {
	AM_spaceBarAdds = flag;
}

- (BOOL)returnKeyAdds {
	return AM_returnKeyAdds;
}

- (void)setReturnKeyAdds:(BOOL)flag {
	AM_returnKeyAdds = flag;
}

- (BOOL)deleteKeyRemoves {
	return AM_deleteKeyRemoves;
}

- (void)setDeleteKeyRemoves:(BOOL)flag {
	AM_deleteKeyRemoves = flag;
}

- (BOOL)spaceBarSendsDoubleAction {
	return AM_spaceBarSendsDoubleAction;
}

- (void)setSpaceBarSendsDoubleAction:(BOOL)flag {
	AM_spaceBarSendsDoubleAction = flag;
}

- (BOOL)returnKeySendsDoubleAction {
	return AM_returnKeySendsDoubleAction;
}

- (void)setReturnKeySendsDoubleAction:(BOOL)flag {
	AM_returnKeySendsDoubleAction = flag;
}

@end
