//
//  AMSortingArrayController.h
//  A sorting, rearrangeable NSArrayController subclass
//
//  Created by Andrew Merenbach on 7/24/07.
//  Copyright 2007 Andrew Merenbach. All rights reserved.
//

#import <Cocoa/Cocoa.h>

@protocol AMSortingArrayControllerEntry
+ (NSArray *)copyKeys;
- (id)copyWithZone:(NSZone *)zone;
- (NSDictionary *)dictionaryRepresentation;
@end


@interface AMSortingArrayController : NSArrayController {
    IBOutlet NSTableView *tableView;
	BOOL AM_allowsRowCopy;
}

//- (void)noteContentChanged:(NSNotification *)aNotification;

- (void)objectDidEndEditing:(id)editor;
- (void)add:(id)sender;

@end


@interface AMSortingArrayController (TableViewDataSource)
// table view drag and drop support

- (BOOL)tableView:(NSTableView *)tv
	writeRowsWithIndexes:(NSIndexSet *)rowIndexes
	toPasteboard:(NSPasteboard*)pboard;

- (NSDragOperation)tableView:(NSTableView*)tv
	validateDrop:(id <NSDraggingInfo>)info
	proposedRow:(int)row
	proposedDropOperation:(NSTableViewDropOperation)op;
    
- (BOOL)tableView:(NSTableView*)tv
	acceptDrop:(id <NSDraggingInfo>)info
	row:(int)row
	dropOperation:(NSTableViewDropOperation)op;


// utility methods

- (void)moveObjectsInArrangedObjectsFromIndexes:(NSIndexSet *)indexSet 
	toIndex:(unsigned)index;
- (void)copyObjectsInArrangedObjectsFromIndexes:(NSIndexSet *)indexSet 
	toIndex:(unsigned)index;
- (int)rowsAboveRow:(int)row inIndexSet:(NSIndexSet *)indexSet;

- (BOOL)allowsRowCopy;
- (void)setAllowsRowCopy:(BOOL)flag;

@end

/*@interface NSDictionary (AMSortingArrayControllerAdditions)
- (NSDictionary *)dictionaryRepresentation;
@end*/
