//
//  AMSortingArrayController.m
//  A sorting, rearrangeable NSArrayController subclass
//
//  Created by Andrew Merenbach on 7/24/07.
//  Copyright 2007 Andrew Merenbach. All rights reserved.
//

//  Many thanks to mmalc on Cocoa-Dev for the template used in this code!

#import "AMSortingArrayController.h"

NSString *MovedRowsType = @"MOVED_ROWS_TYPE";
NSString *CopiedRowsType = @"COPIED_ROWS_TYPE";


@implementation AMSortingArrayController

/*- (void)noteContentChanged:(NSNotification *)aNotification {
	[self rearrangeObjects];
}*/

- (id)init {
	if (nil != (self = [super init])) {
		[self setAllowsRowCopy:NO];
	}
	return self;
}

- (id)initWithContent:(id)content {
	if (nil != (self = [super initWithContent:content])) {
		[self setAllowsRowCopy:NO];
	}
	return self;
}

- (id)initWithCoder:(NSCoder *)aDecoder {
	if (nil != (self = [super initWithCoder:aDecoder])) {
		[self setAllowsRowCopy:NO];
	}
	return self;
}

- (void)objectDidEndEditing:(id)editor {
	[super objectDidEndEditing:editor];
	[self rearrangeObjects];
}

- (void)add:(id)sender {
	[super add:sender];
	[self rearrangeObjects];
}

- (void)awakeFromNib
{
	// register for drag and drop
	[tableView registerForDraggedTypes:
		[NSArray arrayWithObjects:CopiedRowsType, MovedRowsType, nil]];
	[tableView setAllowsMultipleSelection:YES];
	[super awakeFromNib];
}

@end

@implementation AMSortingArrayController (TableViewDataSource)

- (BOOL)tableView:(NSTableView *)aTableView
	writeRowsWithIndexes:(NSIndexSet *)rowIndexes
	toPasteboard:(NSPasteboard*)pboard
{
	// declare our own pasteboard types
	NSArray *typesArray = [NSArray arrayWithObjects:CopiedRowsType, MovedRowsType, nil];
	[pboard declareTypes:typesArray owner:self];
	
	// add row index set for local move
	NSData *archive = [NSKeyedArchiver archivedDataWithRootObject:rowIndexes];
	[pboard setData:archive forType:MovedRowsType];
	[pboard setData:archive forType:CopiedRowsType];
	
	//NSArray *rowCopies = [[self arrangedObjects] objectsAtIndexes:rowIndexes];
	// create new array of selected rows for remote drop
	// could do deferred provision, but keep it direct for clarity
	
	// setPropertyList works here because we're using dictionaries, strings,
	// and dates; otherwise, archive collection to NSData...
	//[pboard setPropertyList:rowCopies forType:CopiedRowsType];
	
	
	/*{
		NSMutableArray *rowStrings = [[NSMutableArray alloc] initWithCapacity:[rowCopies count]];
		NSEnumerator *e = [rowCopies objectEnumerator];
		id rowCopy;
		
		while (rowCopy = [e nextObject]) {
			[rowStrings addObject:[rowCopy valueForKey:@"summary"]];
		}
		
		NSString *combinedRowStrings = [rowStrings componentsJoinedByString:@"\n"];
		[pboard setString:combinedRowStrings forType:NSStringPboardType];
		[rowStrings release];
	}*/
	
	return YES;
}

- (NSDragOperation)tableView:(NSTableView*)tv
	validateDrop:(id <NSDraggingInfo>)info
	proposedRow:(int)row
	proposedDropOperation:(NSTableViewDropOperation)op
{
	NSDragOperation dragOp = NSDragOperationCopy;
	
	// if drag source is self, it's a move
	if ([info draggingSource] == tableView) {
		NSEvent *currentEvent = [NSApp currentEvent];
		int optionKeyPressed = [currentEvent modifierFlags] & NSAlternateKeyMask;
		if (optionKeyPressed == 0) {
			dragOp = NSDragOperationMove;
		}
	}
	// we want to put the object at, not over,
	// the current row (contrast NSTableViewDropOn)
	[tv setDropRow:row dropOperation:NSTableViewDropAbove];
	
	return dragOp;
}

- (BOOL)tableView:(NSTableView*)tv
	acceptDrop:(id <NSDraggingInfo>)info
	row:(int)row
	dropOperation:(NSTableViewDropOperation)op
{
	BOOL success = NO;
	
	if (row < 0) {
		row = 0;
	}
	
	// if drag source is self, it's a move
	if ([info draggingSource] == tableView) {
		[self setSortDescriptors:nil];
		
		NSEvent *currentEvent = [NSApp currentEvent];
		int optionKeyPressed = [currentEvent modifierFlags] & NSAlternateKeyMask;
		
		NSData *archive;
		NSIndexSet *indexSet;
		NSRange range;
		
		if (optionKeyPressed == 0) {
			archive = [[info draggingPasteboard] dataForType:MovedRowsType];
			indexSet = [NSKeyedUnarchiver unarchiveObjectWithData:archive];

			[self moveObjectsInArrangedObjectsFromIndexes:indexSet toIndex:row];
		
			// set selected rows to those that were just moved
			// Need to work out what moved where to determine proper selection...
			int rowsAbove = [self rowsAboveRow:row inIndexSet:indexSet];
			range = NSMakeRange(row - rowsAbove, [indexSet count]);
			
			indexSet = [NSIndexSet indexSetWithIndexesInRange:range];
			[self setSelectionIndexes:indexSet];
			
			success = YES;
		
		} else if ([self allowsRowCopy]) {
			archive = [[info draggingPasteboard] dataForType:CopiedRowsType];
			indexSet = [NSKeyedUnarchiver unarchiveObjectWithData:archive];
			
			// set selected rows to the results of the copy operation
			[self copyObjectsInArrangedObjectsFromIndexes:indexSet toIndex:row];
			range = NSMakeRange(row, [indexSet count]);
			
			indexSet = [NSIndexSet indexSetWithIndexesInRange:range];
			[self setSelectionIndexes:indexSet];
		
			success = YES;
		} else {
			success = NO;
		}
		
		
		//success = YES;
	}/* else {
		// Can we get rows from another document?  If so, add them, then return.
		NSArray *newRows = [[info draggingPasteboard] propertyListForType:CopiedRowsType];
		
		if (newRows) {
			[self setSortDescriptors:nil];
		
			NSRange range = NSMakeRange(row, [newRows count]);
			NSIndexSet *indexSet = [NSIndexSet indexSetWithIndexesInRange:range];
			
			[self insertObjects:newRows atArrangedObjectIndexes:indexSet];
			// set selected rows to those that were just copied
			[self setSelectionIndexes:indexSet];
			success = YES;
		}
	}*/
	
	return success;
}



- (void)moveObjectsInArrangedObjectsFromIndexes:(NSIndexSet *)indexSet
	toIndex:(unsigned int)insertIndex
{
	NSArray		*objects = [self arrangedObjects];
	int			index = [indexSet lastIndex];
	
	int			aboveInsertIndexCount = 0;
	id			object;
	int			removeIndex;
	
	while (NSNotFound != index) {
		if (index >= insertIndex) {
			removeIndex = index + aboveInsertIndexCount;
			aboveInsertIndexCount += 1;
		}
		else {
			removeIndex = index;
			insertIndex -= 1;
		}
		object = [objects objectAtIndex:removeIndex];
		
		[object retain];
		[self removeObjectAtArrangedObjectIndex:removeIndex];
		[self insertObject:object atArrangedObjectIndex:insertIndex];
		[object release];
		
		index = [indexSet indexLessThanIndex:index];
	}
}

- (void)copyObjectsInArrangedObjectsFromIndexes:(NSIndexSet *)indexSet
	toIndex:(unsigned int)insertIndex
{
	NSArray		*objects = [self arrangedObjects];
	int			index = [indexSet lastIndex];
	
	int			aboveInsertIndexCount = 0;
	id			object;
	id			newObject;
	int			copyIndex;
	
	NSDictionary	*attributes;
		
	while (NSNotFound != index) {
		if (index >= insertIndex) {
			copyIndex = index + aboveInsertIndexCount;
			aboveInsertIndexCount += 1;
		}
		else {
			copyIndex = index;
			//insertIndex -= 1;
		}
		object = [objects objectAtIndex:copyIndex];
		attributes = [object dictionaryRepresentation];

		newObject = [self newObject];
		[newObject setValuesForKeysWithDictionary:attributes];
		[self insertObject:newObject atArrangedObjectIndex:insertIndex];
		[newObject release];
		
		index = [indexSet indexLessThanIndex:index];
	}
}

- (int)rowsAboveRow:(int)row inIndexSet:(NSIndexSet *)indexSet
{
	unsigned currentIndex = [indexSet firstIndex];
	int i = 0;
	while (currentIndex != NSNotFound) {
		if (currentIndex < row) { i++; }
		currentIndex = [indexSet indexGreaterThanIndex:currentIndex];
	}
	return i;
}

- (int)numberOfRowsInTableView:(NSTableView *)aTableView
{
	return 0;
}

- (id)tableView:(NSTableView *)aTableView
	objectValueForTableColumn:(NSTableColumn *)aTableColumn
	row:(int)rowIndex
{
	return nil;
}


- (BOOL)allowsRowCopy {
	return AM_allowsRowCopy;
}

- (void)setAllowsRowCopy:(BOOL)flag {
	AM_allowsRowCopy = flag;
}

@end

/*@implementation NSDictionary (AMSortingArrayControllerAdditions)

- (NSDictionary *)dictionaryRepresentation {
	return [[self copy] autorelease];
}

@end*/
